"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isMatchWholeWord = exports.getSchemaName = exports.toPascalCase = exports.template = exports.isTypeAny = exports.getParametersInfo = exports.getParamString = exports.getDefineParam = exports.isAscending = exports.getRefName = exports.getTsType = exports.generateServiceName = exports.getHeaderParams = exports.getPathParams = exports.majorVersionsCheck = exports.getCurrentUrl = void 0;
const jsdoc_1 = require("./utilities/jsdoc");
const child_process_1 = require("child_process");
function getPathParams(parameters) {
    return ((parameters === null || parameters === void 0 ? void 0 : parameters.filter(({ in: In }) => {
        return In === "path";
    })) || []);
}
exports.getPathParams = getPathParams;
function getHeaderParams(parameters, config) {
    const queryParamsArray = (parameters === null || parameters === void 0 ? void 0 : parameters.filter(({ in: In, name }) => {
        var _a, _b;
        return In === "header" && !((_b = (_a = config.ignore) === null || _a === void 0 ? void 0 : _a.headerParams) === null || _b === void 0 ? void 0 : _b.includes(name));
    })) || [];
    const params = getObjectType(queryParamsArray, config);
    return {
        params,
        isNullable: queryParamsArray.every(({ schema = {} }) => !schema.required),
    };
}
exports.getHeaderParams = getHeaderParams;
function toPascalCase(str) {
    return `${str.substring(0, 1).toUpperCase()}${str.substring(1)}`;
}
exports.toPascalCase = toPascalCase;
function replaceWithUpper(str, sp) {
    let pointArray = str.split(sp);
    pointArray = pointArray.map((point) => toPascalCase(point));
    return pointArray.join("");
}
function generateServiceName(endPoint, method, operationId, config) {
    const { methodName, methodParamsByTag, prefix = "" } = config;
    const _endPoint = endPoint.replace(new RegExp(`^${prefix}`, "i"), "");
    let endPointArr = _endPoint.split("/");
    let paramsCount = 0;
    endPointArr = endPointArr.map((value) => {
        if (value.includes("{")) {
            return methodParamsByTag
                ? `P${paramsCount++}`
                : toPascalCase(value.replace("{", "").replace("}", ""));
        }
        return replaceWithUpper(value, "-");
    });
    const path = endPointArr.join("");
    const methodNameTemplate = getTemplate(methodName, operationId);
    const serviceName = template(methodNameTemplate, Object.assign({ path,
        method }, (operationId ? { operationId } : {})));
    return serviceName;
}
exports.generateServiceName = generateServiceName;
function getTemplate(methodName, operationId) {
    const defaultTemplate = "{method}{path}";
    if (!methodName) {
        return defaultTemplate;
    }
    const hasMethodNameOperationId = /(\{operationId\})/i.test(methodName);
    if (hasMethodNameOperationId) {
        return operationId ? methodName : defaultTemplate;
    }
    return methodName;
}
const TYPES = {
    integer: "number",
    number: "number",
    boolean: "boolean",
    object: "object",
    string: "string",
    array: "array",
};
function getDefineParam(name, required = false, schema, config, description) {
    return getParamString(name, required, getTsType(schema, config), description);
}
exports.getDefineParam = getDefineParam;
function getParamString(name, required = false, type, description, isPartial) {
    return `${jsdoc_1.getJsdoc({
        description,
    })}${name}${required ? "" : "?"}: ${isPartial ? `Partial<${type}>` : type}`;
}
exports.getParamString = getParamString;
function getTsType(schema, config) {
    if (isTypeAny(schema)) {
        return "any";
    }
    const { type, $ref, enum: Enum, items, properties, oneOf, additionalProperties, required, allOf, } = schema;
    if ($ref) {
        const refArray = $ref.split("/");
        if (refArray[refArray.length - 2] === "requestBodies") {
            return `RequestBody${getRefName($ref)}`;
        }
        else {
            return getRefName($ref);
        }
    }
    if (Enum) {
        return `${Enum.map((t) => `"${t}"`).join(" | ")}`;
    }
    if (items) {
        return `${getTsType(items, config)}[]`;
    }
    let result = "";
    if (properties) {
        result += getObjectType(Object.entries(properties).map(([pName, _schema]) => ({
            schema: Object.assign(Object.assign({}, _schema), { nullable: config._isSwagger2
                    ? required
                        ? !required.includes(pName)
                        : true
                    : _schema.nullable }),
            name: pName,
        })), config);
    }
    if (oneOf) {
        result = `${result} & (${oneOf
            .map((t) => `(${getTsType(t, config)})`)
            .join(" | ")})`;
    }
    if (allOf) {
        result = `${result} & (${allOf
            .map((_schema) => getTsType(_schema, config))
            .join(" & ")})`;
    }
    if (type === "object" && !result) {
        if (additionalProperties) {
            return `{[x: string]: ${getTsType(additionalProperties, config)}}`;
        }
        return "{[x in string | number ]: any}";
    }
    return result || TYPES[type];
}
exports.getTsType = getTsType;
function getObjectType(parameter, config) {
    const object = parameter
        .sort(({ name, schema: { nullable } = {} }, { name: _name, schema: { nullable: _nullable } = {} }) => {
        if (!nullable && _nullable) {
            return -1;
        }
        else if (nullable && !_nullable) {
            return 1;
        }
        return isAscending(name, _name);
    })
        .reduce((prev, { schema: { deprecated, "x-deprecatedMessage": deprecatedMessage, example, nullable, } = {}, schema, name, }) => {
        return `${prev}${jsdoc_1.getJsdoc(Object.assign(Object.assign({}, schema), { deprecated: deprecated || deprecatedMessage ? deprecatedMessage : undefined, example }))}"${name}"${nullable ? "?" : ""}: ${getTsType(schema, config)};`;
    }, "");
    return object ? `{${object}}` : "";
}
function getSchemaName(name) {
    const removeDot = replaceWithUpper(name, ".");
    const removeBackTick = replaceWithUpper(removeDot, "`");
    const removeFirstBracket = replaceWithUpper(removeBackTick, "[");
    const removeLastBracket = replaceWithUpper(removeFirstBracket, "]");
    return removeLastBracket;
}
exports.getSchemaName = getSchemaName;
function getRefName($ref) {
    const parts = $ref.split("/").pop();
    return getSchemaName(parts || "");
}
exports.getRefName = getRefName;
function isAscending(a, b) {
    if (a > b) {
        return 1;
    }
    if (b > a) {
        return -1;
    }
    return 0;
}
exports.isAscending = isAscending;
function getParametersInfo(parameters, type) {
    const params = (parameters === null || parameters === void 0 ? void 0 : parameters.filter(({ in: In }) => {
        return In === type;
    })) || [];
    return {
        params,
        exist: params.length > 0,
        isNullable: params.every(({ schema }) => !(schema === null || schema === void 0 ? void 0 : schema.required)),
    };
}
exports.getParametersInfo = getParametersInfo;
function majorVersionsCheck(expectedV, inputV) {
    if (!inputV) {
        throw new Error(`Swagger-Typescript working with openApi v3/ swagger v2, seem your json is not openApi openApi v3/ swagger v2`);
    }
    const expectedVMajor = expectedV.split(".")[0];
    const inputVMajor = inputV.split(".")[0];
    function isValidPart(x) {
        return /^\d+$/.test(x);
    }
    if (!isValidPart(expectedVMajor) || !isValidPart(inputVMajor)) {
        throw new Error(`Swagger-Typescript working with openApi v3/ swagger v2 your json openApi version is not valid "${inputV}"`);
    }
    const expectedMajorNumber = Number(expectedVMajor);
    const inputMajorNumber = Number(inputVMajor);
    if (expectedMajorNumber <= inputMajorNumber) {
        return;
    }
    throw new Error(`Swagger-Typescript working with openApi v3/ swagger v2 your json openApi version is ${inputV}`);
}
exports.majorVersionsCheck = majorVersionsCheck;
function isTypeAny(type) {
    if (type === true) {
        return true;
    }
    if (typeof type === "object" && Object.keys(type).length <= 0) {
        return true;
    }
    if (!type || type.AnyValue) {
        return true;
    }
    return false;
}
exports.isTypeAny = isTypeAny;
/** Used to replace {name} in string with obj.name */
function template(str, obj = {}) {
    Object.entries(obj).forEach(([key, value]) => {
        const re = new RegExp(`{${key}}`, "i");
        str = str.replace(re, value);
    });
    const re = new RegExp("{*}", "g");
    if (re.test(str)) {
        throw new Error(`methodName: Some A key is missed "${str}"`);
    }
    return str;
}
exports.template = template;
function isMatchWholeWord(stringToSearch, word) {
    return new RegExp("\\b" + word + "\\b").test(stringToSearch);
}
exports.isMatchWholeWord = isMatchWholeWord;
async function getCurrentUrl({ url, branch: branchName }) {
    var _a, _b;
    const urls = url;
    if (!branchName) {
        branchName = await execAsync("git branch --show-current");
        branchName = branchName === null || branchName === void 0 ? void 0 : branchName.split("/")[0];
        branchName = (_a = urls.find((item) => branchName === item.branch)) === null || _a === void 0 ? void 0 : _a.branch;
    }
    if (!branchName) {
        branchName = (await getSourceBranch()).find((treeItem) => urls.find((item) => treeItem === item.branch));
    }
    const currentUrl = ((_b = urls.find((item) => branchName === item.branch)) === null || _b === void 0 ? void 0 : _b.url) || urls[0].url;
    return currentUrl;
}
exports.getCurrentUrl = getCurrentUrl;
async function getSourceBranch() {
    const result = await execAsync('git log --format="%D"');
    const branchesTree = result
        .split("\n")
        .flatMap((item) => item.split(", "))
        .map((branch) => {
        branch = branch.trim();
        branch = branch.replace("HEAD -> ", "");
        branch = branch.trim();
        return branch;
    });
    return branchesTree;
}
async function execAsync(command) {
    return new Promise((resolve, reject) => {
        const child = child_process_1.exec(command, (error, stdout) => {
            child.kill();
            if (error) {
                reject(error);
                return;
            }
            resolve(stdout);
        });
    });
}
//# sourceMappingURL=utils.js.map